package be.iminds.ilabt.jfed.highlevel.jobs;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.experiment.ExperimentPart;
import be.iminds.ilabt.jfed.experiment.SfaExperimentPart;
import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.highlevel.DesiredStatus;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.tasks.HighLevelTaskFactory;
import be.iminds.ilabt.jfed.highlevel.tasks.ReloadOSTask;
import be.iminds.ilabt.jfed.highlevel.tasks.ShareLanTask;
import be.iminds.ilabt.jfed.lowlevel.authority.finder.AuthorityFinder;
import be.iminds.ilabt.jfed.rspec_fx.model.javafx_impl.FXRspecLink;
import be.iminds.ilabt.jfed.util.common.GeniUrn;

import javax.annotation.Nonnull;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * User: twalcari
 * Date: 12/2/2015
 * Time: 11:24
 */
public class ShareLanJob extends AbstractJob<Void> {
    @Nonnull
    private final FXRspecLink link;
    @Nonnull
    private final String sharedLanName;
    private final boolean share;

    private final AuthorityFinder authorityFinder;

    public ShareLanJob(Experiment experiment, @Nonnull FXRspecLink link, @Nonnull String sharedLanName, boolean share,
                       HighLevelTaskFactory hltf, TaskThread tt, AuthorityFinder authorityFinder) {
        super((share ? "Sharing" : "Unsharing") + " link " + link.getClientId() + " with ID '" + sharedLanName + "'",
                experiment, hltf, tt);
        this.link = link;
        this.sharedLanName = sharedLanName;
        this.share = share;
        this.authorityFinder = authorityFinder;
    }

    @Override
    public Void execute() throws Exception {
        Set<Server> servers = link.getComponentManagerUrns().stream()
                .map(u -> authorityFinder.findByUrn(u, AuthorityFinder.Purpose.CREATE_SLIVER))
                .collect(Collectors.toSet());

        for (Server server : servers) {
            final ShareLanState shareLanState = new ShareLanState(share, server);
            setAndRunState(shareLanState);
        }

        return null;
    }

    private class ShareLanState extends SingleCallState<Task> {
        protected ShareLanState(boolean share, Server server) {
            super(String.format((share ? "S" : "Uns")+"hare LAN at %s", server.getDefaultComponentManagerUrn()),
                    share ?
                            hltf.getShareLanTask(experiment.getSliceOrNull(), server, link.getClientId(), sharedLanName) :
                            hltf.getUnshareLanTask(experiment.getSliceOrNull(), server, link.getClientId(), sharedLanName)
            );
        }
    }
}
